# This script controls the DrivExpress Bus Functional Model
# Make sure you specify the correct location of your license file below


#==============================================================================
# Import all the DrivExpress functions for use in this script
#==============================================================================
from dxpress import *

#==============================================================================
# Set up the basic simulation control settings
#==============================================================================
sim = SimControl()  # create simulation control instance

#-------------------------------
# Location path of licence file
#

####### CHANGE BELOW TO MATCH THE LOCATION OF YOUR LICENSE FILE!! #############

#-------------------------------
# For Linux, use '/' character as directory separator.
# sim.license_file = "/opt/drivexpress/drivexpress_lic_enc.bin"
#
# For Windows, use '/' instead of '\' character as directory separator.
sim.license_file = "./drivexpress_lic_enc.bin"

# Command are created by this script very quickly.  If you want a delay between
#  command execution, you can change the interval.  If this variable is set to 
#  zero, all the commands will get generated and queued until some kind of wait 
#  command is encountered.  Otherwise the script will wait the number of clocks
#  designated by this variable between commands (but will still wait on any 
#  wait commands encountered)
sim.cmd_interval_clks = 0

#==============================================================================
# Set up the PCIe Root Complex etting
#==============================================================================
pcie = PcieRootComplex()  # create PCIe RC instance

# Set the addressing type to 32 or 64 bit.  It's important to get this right
#  because a 32-bit endpoint will not execute 64-bit commands
pcie.is_report_ltssm   = False
pcie.is_report_init_fc = False
pcie.is_64bit_address  = False

# Turn off reporting of the link state machine and initialization.  We're going
#  to use a little script to report the vital info to us instead.

#==============================================================================
# Set up Log Files
#==============================================================================
sim.log_file("drivexpress.log")  # if appned --> sim.log_file("FILE", "app")

#==============================================================================
# Set up a function to watch the link state, which also gets used below 
#  to wait for the link to come up.  This is done using a separate script
#  that we call here.  This script waits for the link to come up before
#  it returns and the following commands execute.
#==============================================================================
sim.run_file("test_pcie_chain_dma_link.py")

#==============================================================================
# Now configure the PCIe endpoint.  For our simple example design, we just have
#  to enable the endpoint memory space so it will accept reads and writes.  We
#  do this by writing to the configuration space.  Before that, just for fun,
#  we check the Device and Vendor ID of the endpoint.
#==============================================================================

# Check VENDOR and DEVICE ID (PCIe configuration space) 
pcie.cfg_read16(VENDOR_ID, 0x1172) #this is the same as pcie.cfg_read16(0x0, 0x1172), which also checks that the vendor ID is 0x1172
pcie.cfg_read16(DEVICE_ID, 0xe001) #this is the same as pcie.cfg_read16(0x2, 0x0004), which also checks that the device ID is 0x0004

# We don't necessarily have to wait for stuff, but before proceeding, lets
#  wait to actually get the read responses back from the endpoint.  This 
#  command waits until all outstanding non-posted requests have completed
#  before allowing the script to continue
pcie.completion_wait()  # Wait for completion for config read/write TLP

# Enable the address space.  This writes to a configuration register to turn
#  on the the ability to read/write to addresses in the endpoint.  If you don't
#  do this, not much will work.  If we had a more complex endpoint, there 
#  would be a bit more to do.
#  This command is the same as pcie.cfg_write16(0x4, 0x46), but it uses
#  some built-in defines to make the command more readable.
pcie.cfg_write16(COMMAND, (PERR_RESPONSE     |
                           BUS_MASTER_ENABLE |
                           MEM_SPACE_ENABLE))
pcie.completion_wait()  # Wait for completion for config read/write TLP

#==============================================================================
# Now do some basic reads and writes to addressable locations in the endpoint.
#==============================================================================

# Write some values to memory
pcie.mem_write32(BAR0, 0x55aa55aa)
pcie.mem_write32(BAR0 + 4, 0xFA55AA5F)
pcie.mem_write32(BAR0 + 8, 0xDEADBEEF)
pcie.mem_write32(BAR0 + 12, 0xCAFEBABE)

pcie.completion_wait()  # Wait for completion for config read/write TLP

# Read some values back from memory
pcie.mem_read32(BAR0, 0x55aa55aa)  
pcie.mem_read32(BAR0 + 4, 0xFA55AA5F)  
pcie.mem_read32(BAR0 + 8, 0xDEADBEEF)
pcie.mem_read32(BAR0 + 12, 0xCAFEBABE)

pcie.completion_wait()  # Wait for completion for config read/write TLP

#print out the test stats
sim.stats()

#stop the simulation
sim.stop()
